package org.apache.maven.continuum.execution.maven.m2;

/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.metadata.ArtifactMetadata;
import org.apache.maven.continuum.execution.AbstractBuildExecutor;
import org.apache.maven.continuum.execution.ContinuumBuildExecutionResult;
import org.apache.maven.continuum.execution.ContinuumBuildExecutor;
import org.apache.maven.continuum.execution.ContinuumBuildExecutorException;
import org.apache.maven.continuum.model.project.BuildDefinition;
import org.apache.maven.continuum.model.project.Project;
import org.apache.maven.continuum.utils.shell.FileIO;
import org.apache.maven.model.Dependency;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.apache.maven.project.artifact.ProjectArtifactMetadata;
import org.codehaus.plexus.util.StringUtils;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
 * @version $Id: MavenTwoBuildExecutor.java 389733 2006-03-29 09:51:42Z evenisse $
 */
public class MavenTwoBuildExecutor
    extends AbstractBuildExecutor
    implements ContinuumBuildExecutor
{
    // ----------------------------------------------------------------------
    //
    // ----------------------------------------------------------------------

    public static final String CONFIGURATION_GOALS = "goals";

    public static final String ID = "maven2";

    // ----------------------------------------------------------------------
    //
    // ----------------------------------------------------------------------     

    /**
     * @plexus.requirement
     */
    private MavenBuilderHelper builderHelper;

    /**
     * @plexus.requirement
     */
    private MavenProjectHelper projectHelper;

    // ----------------------------------------------------------------------
    //
    // ----------------------------------------------------------------------

    public MavenTwoBuildExecutor()
    {
        super( ID, true );
    }

    // ----------------------------------------------------------------------
    // ContinuumBuilder Implementation
    // ----------------------------------------------------------------------

    public ContinuumBuildExecutionResult build( Project project, BuildDefinition buildDefinition, File buildOutput )
        throws ContinuumBuildExecutorException
    {        
        // TODO: get from installation
//        String executable = project.getExecutable();
        String executable = "mvn";

        String arguments = "";

        String buildFile = StringUtils.clean( buildDefinition.getBuildFile() );

        if ( !StringUtils.isEmpty( buildFile ) && !"pom.xml".equals( buildFile ) )
        {
            arguments = "-f " + buildFile + " ";
        }

        arguments += StringUtils.clean( buildDefinition.getArguments() ) + " "
                + StringUtils.clean( buildDefinition.getGoals() );

        ContinuumBuildExecutionResult result = null;
        try {
            if (project.isDerived()) {
                result = buildDerivedMavenProject( project, executable, arguments, buildOutput );
            } else {
                getLogger().info("This is not a derived project, so no copying is performed");
                result = executeShellCommand( project, executable, arguments, buildOutput );
            }            
        } catch (IOException e) {
            e.printStackTrace();
            //getLogger().error("FileIO failed, so no derived project was built.");
        } catch (MavenBuilderHelperException e) {
            e.printStackTrace();
            getLogger().error("builderHelper failed to create MavenProject");
        }
        return result;
    }

    private ContinuumBuildExecutionResult buildDerivedMavenProject(Project project, String executable, 
        String arguments, File buildOutput)
            throws IOException, MavenBuilderHelperException, ContinuumBuildExecutorException {
        //Erik
        ContinuumBuildExecutionResult result = null;
        File workingDirectory = getWorkingDirectory( project );
        String pomPath = workingDirectory + File.separator + "pom.xml";
        String pomBackupPath = workingDirectory + File.separator + "pom.xml.bak";

        //getLogger().info("pomPath: "+ pomPath);
        //getLogger().info("pomBackupPath: " + pomBackupPath);
        getLogger().info("Making av copy of " + pomPath + " to " + pomBackupPath);

        //TODO Decide how to handle IOException
        FileIO.copy(pomPath, pomBackupPath);

        //getLogger().debug("File backup done");

        //getLogger().info("pomPath: " + pomPath);
        MavenProject mavenProject = builderHelper.getMavenProject(new File(pomPath));

        /*
        List<Dependency> dependencies = null;
        if (mavenProject != null) {

            getLogger().debug("Name: " + mavenProject.getName());
            getLogger().debug(mavenProject.getArtifactId() + ":"
                + mavenProject.getGroupId() + ":"
                + mavenProject.getVersion());

            dependencies = mavenProject.getDependencies();

            if (dependencies != null) {
                getLogger().debug("");
                getLogger().debug("===Dependencies BEFORE change===");
                dependencies = mavenProject.getDependencies();
                for (Dependency element : dependencies) {
                    getLogger().debug(element.getGroupId() + ":" + element.getArtifactId() + ":" + element.getVersion());                   
                }
            } else {
                getLogger().info("dependencies was null");
            }
        } else {
            getLogger().info("mavenProject was null");
        }
        */

        List<Dependency> updatedDependencyList = builderHelper.createNewDependencyList(
                mavenProject.getDependencies(), project.getDependencies());

        mavenProject.setDependencies(updatedDependencyList);
        /*
        getLogger().debug("");
        getLogger().debug("===Dependencies AFTER change===");
        dependencies = mavenProject.getDependencies();
        getLogger().debug("dependencies.size: " + dependencies.size());
        for (Dependency element : dependencies) {
            getLogger().debug(element.getGroupId() + ":" + element.getArtifactId() + ":" + element.getVersion());
        }
        */

        PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(pomPath)));
        mavenProject.writeModel(out);
                
        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Making a copy of the modified pom.xml");
            FileIO.copy(pomPath, pomPath + ".lastBuild");
        }

        result = executeShellCommand( project, executable, arguments, buildOutput );

        //FileIO.copy(pomBackupPath, pomPath);
        //FileIO.delete(pomBackupPath);
        FileIO.move(pomBackupPath, pomPath);
        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Original pom.xml was rolled back");
        }                
        return result;
    }

    public void updateProjectFromCheckOut( File workingDirectory, Project project, BuildDefinition buildDefinition )
        throws ContinuumBuildExecutorException
    {
        File f = getPomFile( buildDefinition, workingDirectory );

        if ( !f.exists() )
        {
            throw new ContinuumBuildExecutorException( "Could not find Maven project descriptor." );
        }

        try
        {
            builderHelper.mapMetadataToProject( f, project );
        }
        catch ( MavenBuilderHelperException e )
        {
            throw new ContinuumBuildExecutorException( "Error while mapping metadata.", e );
        }
    }

    private static File getPomFile( BuildDefinition buildDefinition, File workingDirectory )
    {
        File f = null;

        if ( buildDefinition != null )
        {
            String buildFile = StringUtils.clean( buildDefinition.getBuildFile() );

            if ( !StringUtils.isEmpty( buildFile ) )
            {
                f = new File( workingDirectory, buildFile );
            }
        }

        if ( f == null )
        {
            f = new File( workingDirectory, "pom.xml" );
        }

        return f;
    }

    public List getDeployableArtifacts( File workingDirectory, BuildDefinition buildDefinition )
        throws ContinuumBuildExecutorException
    {
        File f = getPomFile( buildDefinition, workingDirectory );

        if ( !f.exists() )
        {
            throw new ContinuumBuildExecutorException( "Could not find Maven project descriptor '" + f + "'." );
        }

        MavenProject project;

        try
        {
            project = builderHelper.getMavenProject( f );
        }
        catch ( MavenBuilderHelperException e )
        {
            throw new ContinuumBuildExecutorException(
                "Unable to read the Maven project descriptor '" + f + "': " + e.getMessage(), e );
        }

        List artifacts = new ArrayList( 1 );

        // Maven could help us out a lot more here by knowing how to get the deployment artifacts from a project.
        // TODO: this is currently quite lame

        Artifact artifact = project.getArtifact();

        String projectPackaging = project.getPackaging();

        boolean isPomArtifact = "pom".equals( projectPackaging );

        if ( isPomArtifact )
        {
            artifact.setFile( project.getFile() );
        }
        else
        {
            // Attach pom
            ArtifactMetadata metadata = new ProjectArtifactMetadata( artifact, project.getFile() );

            artifact.addMetadata( metadata );

            String finalName = project.getBuild().getFinalName();

            String filename = finalName + "." + artifact.getArtifactHandler().getExtension();

            String buildDirectory = project.getBuild().getDirectory();

            File artifactFile = new File( buildDirectory, filename );

            artifact.setFile( artifactFile );

            // sources jar
            File sourcesFile = new File( buildDirectory, finalName + "-sources.jar" );

            if ( sourcesFile.exists() )
            {
                projectHelper.attachArtifact( project, "java-source", "sources", sourcesFile );
            }

            // tests sources jar
            File testsSourcesFile = new File( buildDirectory, finalName + "-test-sources.jar" );

            if ( testsSourcesFile.exists() )
            {
                projectHelper.attachArtifact( project, "java-source", "test-sources", testsSourcesFile );
            }

            // javadoc jar
            File javadocFile = new File( buildDirectory, finalName + "-javadoc.jar" );

            if ( javadocFile.exists() )
            {
                projectHelper.attachArtifact( project, "javadoc", "javadoc", javadocFile );
            }

            // client jar
            File clientFile = new File( buildDirectory, finalName + "-client.jar" );

            if ( clientFile.exists() )
            {
                projectHelper.attachArtifact( project, projectPackaging + "-client", "client", clientFile );
            }

            // Tests jar
            File testsFile = new File( buildDirectory, finalName + "-tests.jar" );

            if ( testsFile.exists() )
            {
                projectHelper.attachArtifact( project, "jar", "tests", testsFile );
            }
        }

        if ( artifact.getFile().exists() )
        {
            artifacts.add( artifact );
        }

        return artifacts;
    }
}
