package org.apache.maven.continuum;
/**
 * User: drolsham
 */

import org.apache.maven.continuum.model.project.ProjectDependency;
import org.apache.maven.continuum.project.DependencyGroup;
import org.codehaus.plexus.PlexusTestCase;

import java.util.List;
import java.util.ArrayList;

public class DependencyHelperTest extends PlexusTestCase {
    private DefaultDependencyHelper dependencyHelper;

    private ProjectDependency testDependency10;
    private ProjectDependency testDependency11;
    private ProjectDependency testDependency12;
    
    private ProjectDependency testDependency20;
    private ProjectDependency testDependency21;
    private ProjectDependency testDependency30;
    private ProjectDependency testDependency40;
    private ProjectDependency testDependency41;
    private ProjectDependency testDependency50;
    private ProjectDependency testDependency51;
    private ProjectDependency testDependency60;
    private ProjectDependency testDependency70;
    private ProjectDependency testDependency71;
    private ProjectDependency testDependency80;
    private ProjectDependency testDependency81;    


    private List<ProjectDependency> originalDependencies = new ArrayList<ProjectDependency>();;
    private List<ProjectDependency> newDependencies = new ArrayList<ProjectDependency>();;


    protected void setUp() throws Exception {
        super.setUp();
        dependencyHelper = (DefaultDependencyHelper) lookup( DependencyHelper.ROLE );

        testDependency10 = SupportUtils.createTestDependency( "groupId_A", "artifactId_A", "version1.0" );
        testDependency11 = SupportUtils.createTestDependency( "groupId_A", "artifactId_A", "version1.1" );
        testDependency12 = SupportUtils.createTestDependency( "groupId_A", "artifactId_A", "version1.2" );
        testDependency20 = SupportUtils.createTestDependency( "groupId_B", "artifactId_B", "version2.0" );
        testDependency21 = SupportUtils.createTestDependency( "groupId_B", "artifactId_B", "version2.1" );
        testDependency30 = SupportUtils.createTestDependency( "groupId_C", "artifactId_C", "version3.0" );
        testDependency40 = SupportUtils.createTestDependency( "groupId_D", "artifactId_D", "version4.0" );
        testDependency41 = SupportUtils.createTestDependency( "groupId_D", "artifactId_D", "version4.1" );
        testDependency50 = SupportUtils.createTestDependency( "groupId_E", "artifactId_E", "version5.0" );
        testDependency51 = SupportUtils.createTestDependency( "groupId_E", "artifactId_E", "version5.1" );
        testDependency60 = SupportUtils.createTestDependency( "groupId_F", "artifactId_F", "version6.0" );
        testDependency70 = SupportUtils.createTestDependency( "javax.activation", "activation", "1.1" );
        testDependency71 = SupportUtils.createTestDependency( "javax.activation", "activation", "1.1_derived" );
        testDependency80 = SupportUtils.createTestDependency( "org.springframework", "spring-web", "2.0.2" );        
        testDependency81 = SupportUtils.createTestDependency( "org.springframework", "spring-web", "2.0.2_derived" );

        originalDependencies.add(testDependency10);
        originalDependencies.add(testDependency20);
        originalDependencies.add(testDependency30);
        originalDependencies.add(testDependency40);
        originalDependencies.add(testDependency50);
        originalDependencies.add(testDependency60);
        originalDependencies.add(testDependency70);
        originalDependencies.add(testDependency80);

    }


    public void testCleanDependencies() {
        newDependencies.add(testDependency11);
        List<ProjectDependency> relevantDependencies = dependencyHelper.findRelevantDependencies(originalDependencies, newDependencies);
        assertEquals(2, relevantDependencies.size());

        dependencyHelper.findSimilarDependenciesInList(testDependency11, relevantDependencies);
    }


    public void testMakeDependencyGroupsWithNoDependencies() {
        List<DependencyGroup> groups = dependencyHelper.makeDependencyGroups(newDependencies);

        assertEquals(0, groups.size());
    }
    public void testMakeDependencyGroupsWithOneDependency() {
        newDependencies.add(testDependency11);
        List<DependencyGroup> groups = dependencyHelper.makeDependencyGroups(newDependencies);

        assertEquals(1, groups.size());
        assertTrue(groups.get(0).contains(testDependency11));
    }
    public void testMakeDependencyGroupsWithTwoDifferentDependencies() {
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        List<DependencyGroup> groups = dependencyHelper.makeDependencyGroups(newDependencies);

        assertEquals(2, groups.size());
    }
    public void testMakeDependencyGroupsAddTwoDependenciesToSameGroup() {
        newDependencies.add(testDependency20);
        newDependencies.add(testDependency21);
        List<DependencyGroup> groups = dependencyHelper.makeDependencyGroups(newDependencies);

        assertEquals(1, groups.size());

        assertTrue(groups.get(0).contains(testDependency20));
        assertTrue(groups.get(0).contains(testDependency21));
    }
    public void testMakeDependencyGroupsWithManyDifferentDependencies() {
        newDependencies.add(testDependency10);
        newDependencies.add(testDependency20);
        newDependencies.add(testDependency30);
        newDependencies.add(testDependency40);
        newDependencies.add(testDependency50);
        List<DependencyGroup> groups = dependencyHelper.makeDependencyGroups(newDependencies);
        assertEquals(5, groups.size());

        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        groups = dependencyHelper.makeDependencyGroups(newDependencies);
        assertEquals(5, groups.size());
    }


    public void testExtractUnaffectedDependencies() {
        newDependencies.add(testDependency10);
        newDependencies.add(testDependency20);
        newDependencies.add(testDependency30);
        
        List<ProjectDependency> unaffectedDependencies =
                dependencyHelper.extractUnaffectedDependencies(originalDependencies, newDependencies);

         assertEquals(5, unaffectedDependencies.size());
    }

   public void testFindSimilarDependenciesInList() {

       List<ProjectDependency> similarDependencies =
               dependencyHelper.findSimilarDependenciesInList(testDependency11, originalDependencies);

       assertEquals(2, similarDependencies.size());
   }


    public void testCreateDependencyListsWithNoNewDependencies() {
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);
        
        assertEquals(1, combinations.size());
    }

    public void testCreateDependencyListsWithOneNewDependency() {
        newDependencies.add(testDependency11);
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);

        assertEquals(2, combinations.size());
    }


    public void testCreateDependencyListsWithTwoNewDependencies() {
        assertTrue(newDependencies.isEmpty());
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);

        assertEquals(4, combinations.size());
    }


    public void testCreateDependencyListsWithTwoNewDependencies2() {
        assertTrue(newDependencies.isEmpty());

           
        newDependencies.add(testDependency71);
        newDependencies.add(testDependency81);
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);

        assertEquals(4, combinations.size());
    }


    public void testCreateDependencyListsWithFourNewDependencies() {
        assertTrue(newDependencies.isEmpty());
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        newDependencies.add(testDependency41);
        newDependencies.add(testDependency51);
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);

        assertEquals(16, combinations.size());
    }

    
    public void testCreateDependencyListsWithThreeDifferentVersions() {
        assertTrue(newDependencies.isEmpty());
        //newDependencies.add(testDependency10);
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency12);
        newDependencies.add(testDependency21);        
        List<List<ProjectDependency>> combinations = dependencyHelper.
                getAllDependencyCombinations(originalDependencies, newDependencies);

        assertEquals(6, combinations.size());   //dependency1 with version 1.0, 1.1 and 1.2 can be combined with
                                                //dependency2 with version 2.0 and 2.1 => 6 combinations  
    }
    

    public void testCreateNewDependencyList() {
        List<ProjectDependency> oldDependencies = new ArrayList<ProjectDependency>();
        oldDependencies.add(testDependency10);
        oldDependencies.add(testDependency20);

        newDependencies = dependencyHelper.createNewDependencyList(oldDependencies, testDependency30);
        assertNull(newDependencies);

        newDependencies = dependencyHelper.createNewDependencyList(oldDependencies, testDependency11);
        assertEquals(2, newDependencies.size());
        assertEquals(newDependencies.get(0).getVersion(), testDependency11.getVersion());
    }


    public void testCloneDependency() {
        ProjectDependency dependency = dependencyHelper.cloneDependency(testDependency10);
        SupportUtils.assertDependencyEquals(dependency, testDependency10);
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        testDependency10 = null;
        testDependency11 = null;
        testDependency20 = null;
        testDependency21 = null;
        testDependency30 = null;

        originalDependencies.clear();
        newDependencies.clear();

        dependencyHelper = null;

    }

}