package org.apache.maven.continuum;

import org.apache.maven.continuum.model.project.*;
import org.codehaus.plexus.PlexusTestCase;

import java.util.ArrayList;
import java.util.List;

/**
 * User: drolsham
 */

public class DerivedProjectManagerTest extends PlexusTestCase {
    private DependencyHelper dependencyHelper;
    private DerivedProjectManager derivedProjectManager;

    private ProjectDependency testDependency10;
    private ProjectDependency testDependency11;
    private ProjectDependency testDependency20;
    private ProjectDependency testDependency21;
    private ProjectDependency testDependency30;
    private ProjectDependency testDependency40;
    private ProjectDependency testDependency41;
    private ProjectDependency testDependency50;
    private ProjectDependency testDependency51;
    private ProjectDependency testDependency60;

    private Project testProject1;
    private Project testProject1b;
    private Project testProject2;
    private Project testProject3;

    //private ContinuumStore store;

    private List<ProjectDependency> newDependencies = new ArrayList<ProjectDependency>();


    protected void setUp() throws Exception {
        super.setUp();        
        derivedProjectManager = (DefaultDerivedProjectManager) lookup( DerivedProjectManager.ROLE );
        dependencyHelper = (DefaultDependencyHelper) lookup( DependencyHelper.ROLE );
        //dependentProject = DependentProject.createDependentProject();
        //store = new SimpleContinuumStore();
        //store = (SimpleContinuumStore) lookup( SimpleContinuumStore.ROLE, "simple");
        //defaultDerivedProjectManager = new DefaultDerivedProjectManager(store);


        ProjectGroup defaultProjectGroup =
                SupportUtils.createTestProjectGroup( "Default Group", "The Default Group", "org.apache.maven.test.default" );

        testProject1 = SupportUtils.createTestProject( "artifactId1", 1, "description1", defaultProjectGroup.getGroupId(), "name1",
                "scmUrl1", 1, "url1", "version1", "workingDirectory1" );

        testProject1b = SupportUtils.createTestProject( "artifactId1", 1, "description1", defaultProjectGroup.getGroupId(), "name1",
                "scmUrl1", 1, "url1", "version1", "workingDirectory1" );


        testProject2 = SupportUtils.createTestProject( "artifactId2", 2, "description2", defaultProjectGroup.getGroupId(), "name2",
                "scmUrl2", 1, "url2", "version2", "workingDirectory2" );

        testProject3 = SupportUtils.createTestProject( "artifactId3", 3, "description3", defaultProjectGroup.getGroupId(), "name3",
                        "scmUrl3", 1, "url3", "version3", "workingDirectory3" );        

        testDependency10 = SupportUtils.createTestDependency( "groupId_A", "artifactId_A", "version1.0" );
        testDependency11 = SupportUtils.createTestDependency( "groupId_A", "artifactId_A", "version1.1" );
        testDependency20 = SupportUtils.createTestDependency( "groupId_B", "artifactId_B", "version2.0" );
        testDependency21 = SupportUtils.createTestDependency( "groupId_B", "artifactId_B", "version2.1" );
        testDependency30 = SupportUtils.createTestDependency( "groupId_C", "artifactId_C", "version3.0" );
        testDependency40 = SupportUtils.createTestDependency( "groupId_D", "artifactId_D", "version4.0" );
        testDependency41 = SupportUtils.createTestDependency( "groupId_D", "artifactId_D", "version4.1" );
        testDependency50 = SupportUtils.createTestDependency( "groupId_E", "artifactId_E", "version5.0" );
        testDependency51 = SupportUtils.createTestDependency( "groupId_E", "artifactId_E", "version5.1" );
        testDependency60 = SupportUtils.createTestDependency( "groupId_F", "artifactId_F", "version6.0" );

        testDependency11.setDerived(true);
        testDependency21.setDerived(true);
        testDependency41.setDerived(true);
        testDependency51.setDerived(true);


        testProject1.addDependency(testDependency10);
        testProject1.addDependency(testDependency20);
        testProject1.addDependency(testDependency30);
        testProject1.addDependency(testDependency40);
        testProject1.addDependency(testDependency50);
        testProject1.addDependency(testDependency60);


        defaultProjectGroup.addProject(testProject1);
        //store.addProjectGroup(defaultProjectGroup);
    }


    public void testProjectSameWithDifferentDependencies() {
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency10));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency20));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency30));
        
        assertFalse(derivedProjectManager.projectSame(testProject1, testProject1b));
    }

    public void testProjectSameWithAlmostIdenticalProjects() {
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency10));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency20));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency30));        
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency40));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency50));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency60));

        assertTrue(derivedProjectManager.projectSame(testProject1, testProject1b));
    }

    public void testProjectSameWithOneModifiedDependency() {
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency10));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency20));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency30));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency40));
        testProject1b.addDependency(dependencyHelper.cloneDependency(testDependency50));

        ProjectDependency tmp = dependencyHelper.cloneDependency(testDependency60);
        tmp.setVersion("differentVersion");
        testProject1b.addDependency(tmp);

        assertFalse(derivedProjectManager.projectSame(testProject1, testProject1b));
    }



    /*
    public void testCreateListOfDependentProjects() throws Exception {
        defaultDerivedProjectManager.createListOfProjects(store, testDependency11);
        SupportUtils.assertProjectEquals(defaultDerivedProjectManager.getProjects().get(0).getOriginalProject(), testProject1);

        defaultDerivedProjectManager.createListOfProjects(store, testDependency30);
        SupportUtils.assertProjectEquals(defaultDerivedProjectManager.getProjects().get(0).getOriginalProject(), testProject1);

        defaultDerivedProjectManager.createListOfProjects(store, testDependency20);
        //System.out.println("Size: " + defaultDerivedProjectManager.getDependentProjects().size());
        assertTrue(defaultDerivedProjectManager.getProjects().isEmpty());

    }
    */
    /*
    public void testProjectSameSimple() {
        assertTrue(derivedProjectManager.projectSame(testProject1, testProject1));
        assertFalse(derivedProjectManager.projectSame(testProject1, testProject2));
    }

    public void testProjectSameUnequalNumberOfDependencies() {
        testProject1b.removeDependency(testDependency30);
        assertFalse(derivedProjectManager.projectSame(testProject1, testProject1b));
    }

    public void testProjectSameWithOneModifiedDependency() {
        List<ProjectDependency> list = new ArrayList<ProjectDependency>();
        list.add(testDependency41);
        Project testProject3 = defaultDerivedProjectManager.createProject(testProject1, list);
        assertFalse(defaultDerivedProjectManager.projectSame(testProject1, testProject3));
    }


    public void testCreateDerivedProject() {
        Project testDerivedProject;

        List<ProjectDependency> list = new ArrayList<ProjectDependency>();
        list.add(testDependency41);
        testProject1.removeDependency(testDependency40);
        testDerivedProject = derivedProjectManager.createProject(testProject1, list);
        assertNull(testDerivedProject);

        list = new ArrayList<ProjectDependency>();
        list.add(testDependency11);
        testDerivedProject = derivedProjectManager.createProject(testProject1, list);
        assertNotNull(testDerivedProject);

        //TODO Reference to originalProject is not available
        //SupportUtils.assertProjectSame(testDerivedProject.getOriginalProject(), testProject1);
    }
    */    

    public void testCreateDerivedProjectsWithOneNewDependency() {
        newDependencies.add(testDependency11);
        DependencyHelper dependencyHelper = derivedProjectManager.getDependencyHelper();
        assertNotNull(dependencyHelper);

        List<Project> testProjects =
                derivedProjectManager.createDerivedProjects(testProject1, newDependencies, null);
        assertEquals(1, testProjects.size());

        Project derivedProject = testProjects.get(0);
        ProjectDependency pd; 
        for (Object object : derivedProject.getDependencies()) {
            pd = (ProjectDependency) object;
            if (dependencyHelper.dependenciesEqual(pd, testDependency11)) {
                assertTrue(pd.isDerived());   
            }
        }
    }

    public void testCreateDerivedProjectsWithTwoNewDependencies() {
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        
        List<Project> testProjects =
                derivedProjectManager.createDerivedProjects(testProject1, newDependencies, null);
        assertEquals(3, testProjects.size());

        //assertDerivedPropertyOnDependencies(testProjects);
    }

    public void testCreateDerivedProjectsWithFourNewDependencies() {       
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);
        newDependencies.add(testDependency41);
        newDependencies.add(testDependency51);

        assertNotNull(derivedProjectManager.getDependencyHelper());
        List<Project> testProjects =
                derivedProjectManager.createDerivedProjects(testProject1, newDependencies, null);
        assertEquals(15, testProjects.size());

        //assertDerivedPropertyOnDependencies(testProjects);
    }

    public void testCreateDerivedProjectsWithMultipleProjectsAndMultipleNewDependencies() {
        newDependencies.add(testDependency11);
        newDependencies.add(testDependency21);       
        newDependencies.add(testDependency51);

        testProject2.addDependency(testDependency10);
        testProject2.addDependency(testDependency30);
        testProject2.addDependency(testDependency40);
        testProject2.addDependency(testDependency50);

        testProject3.addDependency(testDependency10);
        testProject3.addDependency(testDependency20);
        testProject3.addDependency(testDependency30);

        List<Project> testProjects =
                derivedProjectManager.createDerivedProjects(testProject1, newDependencies, null);
        assertEquals(7, testProjects.size());
        testProjects.addAll(derivedProjectManager.createDerivedProjects(testProject2, newDependencies, null));
        assertEquals(10, testProjects.size());
        testProjects.addAll(derivedProjectManager.createDerivedProjects(testProject3, newDependencies, null));
        assertEquals(13, testProjects.size());
    }


    public void testCloneDeveloper() {
        ProjectDeveloper testDeveloper1 = SupportUtils.createTestDeveloper( 1, "email1", "name1", "scmId1" );
        ProjectDeveloper developer = derivedProjectManager.cloneDeveloper(testDeveloper1);
        SupportUtils.assertDeveloperEquals(developer, testDeveloper1);
    }

    public void testCloneNotifier() {
        ProjectNotifier testNotifier1 = SupportUtils.createTestNotifier( 11, true, true, false, "type11" );
        ProjectNotifier notifier = derivedProjectManager.cloneNotifier(testNotifier1);
        SupportUtils.assertNotifierEquals(notifier, testNotifier1);        
    }


    protected void tearDown(){
        testDependency10 = null;
        testDependency11 = null;
        testDependency20 = null;
        testDependency21 = null;
        testDependency30 = null;
        testDependency40 = null;
        testDependency41 = null;
        testDependency50 = null;
        testDependency51 = null;
        testDependency60 = null;

        newDependencies.clear();

        testProject1 = null;
        testProject1b = null;
        testProject2 = null;
    }

}
