package org.apache.maven.continuum;

import org.apache.maven.continuum.model.project.*;
import org.apache.maven.continuum.model.system.Installation;
import org.apache.maven.continuum.model.scm.ScmResult;
import org.apache.maven.continuum.model.scm.ChangeSet;
import org.apache.maven.continuum.model.scm.ChangeFile;
import org.apache.maven.continuum.store.ContinuumStoreException;
import org.apache.maven.continuum.store.ContinuumObjectNotFoundException;
import org.apache.maven.continuum.store.ContinuumStore;
import org.codehaus.plexus.jdo.ConfigurableJdoFactory;
import org.codehaus.plexus.jdo.JdoFactory;
import org.codehaus.plexus.jdo.DefaultConfigurableJdoFactory;
import org.jpox.SchemaTool;
import junit.framework.TestCase;

import javax.jdo.JDODetachedFieldAccessException;
import javax.jdo.PersistenceManagerFactory;
import javax.jdo.PersistenceManager;
import java.util.*;
import java.net.URL;

/**
 * Created by IntelliJ IDEA.
 * User: drolsham
 * Date: Feb 15, 2007
 * Time: 7:21:05 PM
 * To change this template use File | Settings | File Templates.
 */
public class SupportUtils extends TestCase {

    public static BuildDefinition createTestBuildDefinition( BuildDefinition buildDefinition )
       {
           return createTestBuildDefinition( buildDefinition.getArguments(), buildDefinition.getBuildFile(),
                                             buildDefinition.getGoals(), buildDefinition.getProfile(),
                                             buildDefinition.getSchedule() );
       }

       public static BuildDefinition createTestBuildDefinition( String arguments, String buildFile, String goals,
                                                                 Profile profile, Schedule schedule )
       {
           BuildDefinition definition = new BuildDefinition();
           definition.setArguments( arguments );
           definition.setBuildFile( buildFile );
           definition.setGoals( goals );
           definition.setProfile( profile );
           definition.setSchedule( schedule );
           return definition;
       }

       public static ProjectNotifier createTestNotifier( ProjectNotifier notifier )
       {
           return createTestNotifier( notifier.getRecipientType(), notifier.isSendOnError(), notifier.isSendOnFailure(),
                                      notifier.isSendOnSuccess(), notifier.getType() );
       }

       public static ProjectNotifier createTestNotifier( int recipientType, boolean sendOnError, boolean sendOnFailure,
                                                          boolean sendOnSuccess, String type )
       {
           Map configuration = new HashMap();
           configuration.put( "key1", "value1" );
           configuration.put( "key2", "value2" );

           ProjectNotifier notifier = new ProjectNotifier();
           notifier.setConfiguration( configuration );
           notifier.setRecipientType( recipientType );
           notifier.setSendOnError( sendOnError );
           notifier.setSendOnFailure( sendOnFailure );
           notifier.setSendOnSuccess( sendOnSuccess );
           notifier.setType( type );

           return notifier;
       }

       public static ScmResult createTestScmResult( ScmResult scmResult, String base )
       {
           return createTestScmResult( scmResult.getCommandOutput(), scmResult.getProviderMessage(), scmResult.isSuccess(),
                                       base );
       }

       public static ScmResult createTestScmResult( String commandOutput, String providerMessage, boolean success,
                                                     String base )
       {
           ScmResult scmResult = new ScmResult();
           scmResult.setCommandOutput( commandOutput );
           scmResult.setProviderMessage( providerMessage );
           scmResult.setSuccess( success );

           List changes = new ArrayList();
           changes.add( createTestChangeSet( "author" + base + ".1", "comment" + base + ".1", base + ".1" ) );
           changes.add( createTestChangeSet( "author" + base + ".2", "comment" + base + ".2", base + ".2" ) );
           scmResult.setChanges( changes );
           return scmResult;
       }

       public static ChangeSet createTestChangeSet( String author, String comment, String base )
       {
           ChangeSet changeSet = new ChangeSet();
           changeSet.setAuthor( author );
           changeSet.setComment( comment );
           changeSet.setDate( System.currentTimeMillis() );
           List files = new ArrayList();
           files.add( createTestChangeFile( "name" + base + ".1", "rev" + base + ".1" ) );
           files.add( createTestChangeFile( "name" + base + ".2", "rev" + base + ".2" ) );
           files.add( createTestChangeFile( "name" + base + ".3", "rev" + base + ".3" ) );
           changeSet.setFiles( files );
           return changeSet;
       }

       public static ChangeFile createTestChangeFile( String name, String revision )
       {
           ChangeFile changeFile = new ChangeFile();
           changeFile.setName( name );
           changeFile.setRevision( revision );
           return changeFile;
       }

       public static BuildResult createTestBuildResult( BuildResult buildResult )
       {
           return createTestBuildResult( buildResult.getTrigger(), buildResult.isSuccess(), buildResult.getState(),
                                         buildResult.getExitCode(), buildResult.getError(), buildResult.getBuildNumber(),
                                         buildResult.getStartTime(), buildResult.getEndTime() );
       }

       public static BuildResult createTestBuildResult( int trigger, boolean success, int state, int exitCode,
                                                         String error, int buildNumber, long startTime, long endTime )
       {
           BuildResult result = new BuildResult();
           result.setBuildNumber( buildNumber );
           result.setStartTime( startTime );
           result.setEndTime( endTime );
           result.setError( error );
           result.setExitCode( exitCode );
           result.setState( state );
           result.setSuccess( success );
           result.setTrigger( trigger );
           return result;
       }

       public static Installation createTestInstallation( String name, String path, String version )
       {
           Installation installation = new Installation();
           installation.setName( name );
           installation.setPath( path );
           installation.setVersion( version );
           return installation;
       }

       public static Installation createTestInstallation( Installation installation )
       {
           return createTestInstallation( installation.getName(), installation.getPath(), installation.getVersion() );
       }

       public static Schedule createTestSchedule( Schedule schedule )
       {
           return createTestSchedule( schedule.getName(), schedule.getDescription(), schedule.getDelay(),
                                      schedule.getCronExpression(), schedule.isActive() );
       }

       public static Schedule createTestSchedule( String name, String description, int delay, String cronExpression,
                                                   boolean active )
       {
           Schedule schedule = new Schedule();
           schedule.setActive( active );
           schedule.setCronExpression( cronExpression );
           schedule.setDelay( delay );
           schedule.setDescription( description );
           schedule.setName( name );
           return schedule;
       }

       public static Profile createTestProfile( Profile profile )
       {
           return createTestProfile( profile.getName(), profile.getDescription(), profile.getScmMode(),
                                     profile.isBuildWithoutChanges(), profile.isActive(), profile.getJdk(),
                                     profile.getBuilder() );
//                                  createTestInstallation( profile.getJdk() ),
//                                  createTestInstallation( profile.getBuilder() ) );
       }

       public static Profile createTestProfile( String name, String description, int scmMode, boolean buildWithoutChanges,
                                                 boolean active, Installation jdk, Installation builder )
       {
           Profile profile = new Profile();
           profile.setActive( active );
           profile.setBuildWithoutChanges( buildWithoutChanges );
           profile.setScmMode( scmMode );
           profile.setDescription( description );
           profile.setName( name );
           profile.setBuilder( builder );
           profile.setJdk( jdk );
           return profile;
       }

       public static ProjectGroup createTestProjectGroup( ProjectGroup group )
       {
           return createTestProjectGroup( group.getName(), group.getDescription(), group.getGroupId() );
       }

       public static ProjectGroup createTestProjectGroup( String name, String description, String groupId )
       {
           ProjectGroup group = new ProjectGroup();
           group.setName( name );
           group.setDescription( description );
           group.setGroupId( groupId );
           return group;
       }

       public static Project createTestProject( Project project )
       {
           return createTestProject( project.getArtifactId(), project.getBuildNumber(), project.getDescription(),
                                     project.getGroupId(), project.getName(), project.getScmUrl(), project.getState(),
                                     project.getUrl(), project.getVersion(), project.getWorkingDirectory() );
       }

       public static Project createTestProject( String artifactId, int buildNumber, String description, String groupId,
                                                 String name, String scmUrl, int state, String url, String version,
                                                 String workingDirectory )
       {
           Project project = new Project();
           project.setArtifactId( artifactId );
           project.setBuildNumber( buildNumber );
           project.setDescription( description );
           project.setGroupId( groupId );
           project.setName( name );
           project.setScmUrl( scmUrl );
           project.setState( state );
           project.setUrl( url );
           project.setVersion( version );
           project.setWorkingDirectory( workingDirectory );
           return project;
       }

       public static void assertProjectEquals( Project retrievedProject, Project project )
       {
           assertNotSame( project, retrievedProject );
           assertProjectSame(retrievedProject, project);          
       }

    //Erik lagt til 
        public static void assertProjectSame( Project retrievedProject, Project project )
       {
           assertEquals( "compare projects", retrievedProject, project );
           //assertNotSame( project, retrievedProject );
           // aggressive compare, as equals is using the identity
           assertEquals( "compare project - name", project.getName(), retrievedProject.getName() );
           assertEquals( "compare project - desc", project.getDescription(), retrievedProject.getDescription() );
           assertEquals( "compare project - groupId", project.getGroupId(), retrievedProject.getGroupId() );
           assertEquals( "compare project - artifactId", project.getArtifactId(), retrievedProject.getArtifactId() );
           assertEquals( "compare project - buildNumber", project.getBuildNumber(), retrievedProject.getBuildNumber() );
           assertEquals( "compare project - scmUrl", project.getScmUrl(), retrievedProject.getScmUrl() );
           assertEquals( "compare project - state", project.getState(), retrievedProject.getState() );
           assertEquals( "compare project - url", project.getUrl(), retrievedProject.getUrl() );
           assertEquals( "compare project - version", project.getVersion(), retrievedProject.getVersion() );
           assertEquals( "compare project - workingDirectory", project.getWorkingDirectory(),
                         retrievedProject.getWorkingDirectory() );
       }


       public static void assertProjectGroupEquals( ProjectGroup retrievedGroup, ProjectGroup group )
       {
           assertEquals( "compare project groups", retrievedGroup, group );
           assertNotSame( group, retrievedGroup );
           // aggressive compare, as equals is using the identity
           assertEquals( "compare project groups - name", group.getName(), retrievedGroup.getName() );
           assertEquals( "compare project groups - desc", group.getDescription(), retrievedGroup.getDescription() );
           assertEquals( "compare project groups - groupId", group.getGroupId(), retrievedGroup.getGroupId() );
       }

       public static void checkProjectGroupDefaultFetchGroup( ProjectGroup retrievedGroup )
       {
           try
           {
               retrievedGroup.getBuildDefinitions();
               fail( "buildDefinitions should not be in the default fetch group" );
           }
           catch ( JDODetachedFieldAccessException expected )
           {
               assertTrue( true );
           }

           try
           {
               retrievedGroup.getNotifiers();
               fail( "notifiers should not be in the default fetch group" );
           }
           catch ( JDODetachedFieldAccessException expected )
           {
               assertTrue( true );
           }
       }

       public static void checkProjectDefaultFetchGroup( Project project )
       {
           checkProjectFetchGroup( project, false, false, false, false );
       }

       public static void checkProjectFetchGroup( Project project, boolean checkoutFetchGroup,
                                                   boolean buildResultsFetchGroup, boolean detailsFetchGroup,
                                                   boolean fineDetailsFetchGroup )
       {
           if ( !fineDetailsFetchGroup )
           {
               try
               {
                   project.getDevelopers();

                   fail( "developers should not be in the default fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
                   assertTrue( true );
               }

               try
               {
                   project.getDependencies();

                   fail( "dependencies should be in the default fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
               }
           }

           if ( !detailsFetchGroup )
           {
               try
               {
                   project.getNotifiers();

                   fail( "notifiers should not be in the default fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
                   assertTrue( true );
               }

               try
               {
                   project.getBuildDefinitions();

                   fail( "buildDefinitions should not be in the default fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
                   assertTrue( true );
               }
           }

           if ( !checkoutFetchGroup )
           {
               try
               {
                   project.getCheckoutResult();

                   fail( "checkoutResult should not be in the fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
                   assertTrue( true );
               }
           }

           if ( !buildResultsFetchGroup )
           {
               try
               {
                   project.getBuildResults();

                   fail( "buildResults should not be in the default fetch group" );
               }
               catch ( JDODetachedFieldAccessException expected )
               {
                   assertTrue( true );
               }
           }
       }

       public static void checkBuildResultDefaultFetchGroup( BuildResult buildResult )
       {
           try
           {
               buildResult.getScmResult();

               fail( "scmResult should not be in the default fetch group" );
           }
           catch ( JDODetachedFieldAccessException expected )
           {
               assertTrue( true );
           }
           // TODO: artifacts
           // TODO: report
           // TODO: long error data
       }

       public static void assertScheduleEquals( Schedule retrievedSchedule, Schedule schedule )
       {
           assertNotSame( schedule, retrievedSchedule );
           assertEquals( "compare schedule - id", schedule.getId(), retrievedSchedule.getId() );
           assertEquals( "compare schedule - name", schedule.getName(), retrievedSchedule.getName() );
           assertEquals( "compare schedule - desc", schedule.getDescription(), retrievedSchedule.getDescription() );
           assertEquals( "compare schedule - delay", schedule.getDelay(), retrievedSchedule.getDelay() );
           assertEquals( "compare schedule - cron", schedule.getCronExpression(), retrievedSchedule.getCronExpression() );
           assertEquals( "compare schedule - active", schedule.isActive(), retrievedSchedule.isActive() );
       }

       public static void assertProfileEquals( Profile retrievedProfile, Profile profile )
       {
           assertNotSame( profile, retrievedProfile );
           assertEquals( "compare profile - name", profile.getName(), retrievedProfile.getName() );
           assertEquals( "compare profile - desc", profile.getDescription(), retrievedProfile.getDescription() );
           assertEquals( "compare profile - scmMode", profile.getScmMode(), retrievedProfile.getScmMode() );
           assertEquals( "compare profile - build w/o changes", profile.isBuildWithoutChanges(),
                         retrievedProfile.isBuildWithoutChanges() );
           assertEquals( "compare profile - active", profile.isActive(), retrievedProfile.isActive() );
       }

       public static void assertInstallationEquals( Installation retrievedInstallation, Installation installation )
       {
           assertEquals( "compare installation - name", installation.getName(), retrievedInstallation.getName() );
           assertEquals( "compare installation - path", installation.getPath(), retrievedInstallation.getPath() );
           assertEquals( "compare installation - version", installation.getVersion(), retrievedInstallation.getVersion() );
       }

       public static void assertBuildResultEquals( BuildResult retrievedBuildResult, BuildResult buildResult )
       {
           assertEquals( "compare build result - build #", buildResult.getBuildNumber(),
                         retrievedBuildResult.getBuildNumber() );
           assertEquals( "compare build result - end time", buildResult.getEndTime(), retrievedBuildResult.getEndTime() );
           assertEquals( "compare build result - error", buildResult.getError(), retrievedBuildResult.getError() );
           assertEquals( "compare build result - exit code", buildResult.getExitCode(),
                         retrievedBuildResult.getExitCode() );
           assertEquals( "compare build result - start time", buildResult.getStartTime(),
                         retrievedBuildResult.getStartTime() );
           assertEquals( "compare build result - state", buildResult.getState(), retrievedBuildResult.getState() );
           assertEquals( "compare build result - trigger", buildResult.getTrigger(), retrievedBuildResult.getTrigger() );
       }

       public static void assertScmResultEquals( ScmResult retrievedScmResult, ScmResult scmResult )
       {
           assertEquals( "compare SCM result - output", scmResult.getCommandOutput(),
                         retrievedScmResult.getCommandOutput() );
           assertEquals( "compare SCM result - message", scmResult.getProviderMessage(),
                         retrievedScmResult.getProviderMessage() );
           assertEquals( "compare SCM result - success", scmResult.isSuccess(), retrievedScmResult.isSuccess() );
           assertEquals( "compare SCM result - changes size", retrievedScmResult.getChanges().size(),
                         scmResult.getChanges().size() );
           for ( int i = 0; i < retrievedScmResult.getChanges().size(); i++ )
           {
               assertChangeSetEquals( (ChangeSet) retrievedScmResult.getChanges().get( i ),
                                      (ChangeSet) scmResult.getChanges().get( i ) );
           }
       }

       public static void assertChangeSetEquals( ChangeSet retrievedChangeSet, ChangeSet changeSet )
       {
           assertEquals( "compare change set result - author", changeSet.getAuthor(), retrievedChangeSet.getAuthor() );
           assertEquals( "compare change set result - comment", changeSet.getComment(), retrievedChangeSet.getComment() );
           //Remove this test, in some case we have a 1ms difference between two dates
           //assertEquals( "compare change set result - date", changeSet.getDate(), retrievedChangeSet.getDate() );
           assertEquals( "compare change set result - files size", retrievedChangeSet.getFiles().size(),
                         changeSet.getFiles().size() );
           for ( int i = 0; i < retrievedChangeSet.getFiles().size(); i++ )
           {
               assertChangeFileEquals( (ChangeFile) retrievedChangeSet.getFiles().get( i ),
                                       (ChangeFile) changeSet.getFiles().get( i ) );
           }
       }

       public static void assertChangeFileEquals( ChangeFile retrievedChangeFile, ChangeFile changeFile )
       {
           assertEquals( "compare change file result - name", retrievedChangeFile.getName(), changeFile.getName() );
           assertEquals( "compare change file result - revision", retrievedChangeFile.getRevision(),
                         changeFile.getRevision() );
       }

       public static void assertNotifiersEqual( List retrievedNotifiers, List notifiers )
       {
           for ( int i = 0; i < retrievedNotifiers.size(); i++ )
           {
               assertNotifierEquals( (ProjectNotifier) retrievedNotifiers.get( i ), (ProjectNotifier) notifiers.get( i ) );
           }
       }

       public static void assertNotifierEquals( ProjectNotifier retrievedNotifier, ProjectNotifier notifier )
       {
           assertEquals( "compare notifier - recipient type", notifier.getRecipientType(),
                         retrievedNotifier.getRecipientType() );
           assertEquals( "compare notifier - type", notifier.getType(), retrievedNotifier.getType() );
           assertEquals( "compare notifier - configuration", notifier.getConfiguration(),
                         retrievedNotifier.getConfiguration() );
           assertEquals( "compare notifier - send on success", notifier.isSendOnSuccess(),
                         retrievedNotifier.isSendOnSuccess() );
           assertEquals( "compare notifier - send on failure", notifier.isSendOnFailure(),
                         retrievedNotifier.isSendOnFailure() );
           assertEquals( "compare notifier - send on error", notifier.isSendOnError(), retrievedNotifier.isSendOnError() );
       }

       public static void assertBuildDefinitionsEqual( List retrievedBuildDefinitions, List buildDefinitions )
       {
           for ( int i = 0; i < retrievedBuildDefinitions.size(); i++ )
           {
               BuildDefinition retrievedBuildDefinition = (BuildDefinition) retrievedBuildDefinitions.get( i );
               BuildDefinition buildDefinition = (BuildDefinition) buildDefinitions.get( i );
               assertBuildDefinitionEquals( retrievedBuildDefinition, buildDefinition );
               assertScheduleEquals( retrievedBuildDefinition.getSchedule(), buildDefinition.getSchedule() );
               assertProfileEquals( retrievedBuildDefinition.getProfile(), buildDefinition.getProfile() );
           }
       }

       public static void assertBuildDefinitionEquals( BuildDefinition retrievedBuildDefinition,
                                                        BuildDefinition buildDefinition )
       {
           assertEquals( "compare build definition - arguments", buildDefinition.getArguments(),
                         retrievedBuildDefinition.getArguments() );
           assertEquals( "compare build definition - build file", buildDefinition.getBuildFile(),
                         retrievedBuildDefinition.getBuildFile() );
           assertEquals( "compare build definition - goals", buildDefinition.getGoals(),
                         retrievedBuildDefinition.getGoals() );
       }

       public static void assertDevelopersEqual( List retrievedDevelopers, List developers )
       {
           for ( int i = 0; i < retrievedDevelopers.size(); i++ )
           {
               assertDeveloperEquals( (ProjectDeveloper) retrievedDevelopers.get( i ),
                                      (ProjectDeveloper) developers.get( i ) );
           }
       }

       public static void assertDeveloperEquals( ProjectDeveloper retrievedDeveloper, ProjectDeveloper developer )
       {
           assertEquals( "compare developer - name", developer.getName(), retrievedDeveloper.getName() );
           assertEquals( "compare developer - email", developer.getEmail(), retrievedDeveloper.getEmail() );
           assertEquals( "compare developer - scmId", developer.getScmId(), retrievedDeveloper.getScmId() );
           assertEquals( "compare developer - continuumId", developer.getContinuumId(),
                         retrievedDeveloper.getContinuumId() );
       }

       public static void assertDependenciesEqual( List retrievedDependencies, List dependencies )
       {
           for ( int i = 0; i < retrievedDependencies.size(); i++ )
           {
               assertDependencyEquals( (ProjectDependency) retrievedDependencies.get( i ),
                                       (ProjectDependency) dependencies.get( i ) );
           }
       }

       public static void assertDependencyEquals( ProjectDependency retrievedDependency, ProjectDependency dependency )
       {
           assertEquals( "compare dependency - groupId", dependency.getGroupId(), retrievedDependency.getGroupId() );
           assertEquals( "compare dependency - artifactId", dependency.getArtifactId(),
                         retrievedDependency.getArtifactId() );
           assertEquals( "compare dependency - version", dependency.getVersion(), retrievedDependency.getVersion() );
       }

       public static ProjectDependency createTestDependency( ProjectDependency dependency )
       {
           return createTestDependency( dependency.getGroupId(), dependency.getArtifactId(), dependency.getVersion() );
       }

       public static ProjectDeveloper createTestDeveloper( ProjectDeveloper developer )
       {
           return createTestDeveloper( developer.getContinuumId(), developer.getEmail(), developer.getName(),
                                       developer.getScmId() );
       }

       public static ProjectDependency createTestDependency( String groupId, String artifactId, String version )
       {
           ProjectDependency dependency = new ProjectDependency();
           dependency.setArtifactId( artifactId );
           dependency.setGroupId( groupId );
           dependency.setVersion( version );
           return dependency;
       }

       public static ProjectDeveloper createTestDeveloper( int continuumId, String email, String name, String scmId )
       {
           ProjectDeveloper developer = new ProjectDeveloper();
           developer.setContinuumId( continuumId );
           developer.setEmail( email );
           developer.setName( name );
           developer.setScmId( scmId );
           return developer;
       }

       /**
        * Setup JDO Factory
        *
        * @todo push down to a Jdo specific test
        */
       /*
       public ContinuumStore createStore()
           throws Exception
       {
           ConfigurableJdoFactory jdoFactory = (ConfigurableJdoFactory) lookup( JdoFactory.ROLE );
           assertEquals( DefaultConfigurableJdoFactory.class.getName(), jdoFactory.getClass().getName() );

           jdoFactory.setPersistenceManagerFactoryClass( "org.jpox.PersistenceManagerFactoryImpl" );

           // TODO: add ability to test with various
           jdoFactory.setDriverName( "org.hsqldb.jdbcDriver" );

           jdoFactory.setUrl( "jdbc:hsqldb:mem:" + getName() );

           jdoFactory.setUserName( "sa" );

           jdoFactory.setPassword( "" );

           jdoFactory.setProperty( "org.jpox.transactionIsolation", "READ_UNCOMMITTED" );

           jdoFactory.setProperty( "org.jpox.poid.transactionIsolation", "READ_UNCOMMITTED" );

           jdoFactory.setProperty( "org.jpox.autoCreateSchema", "true" );

           Properties properties = jdoFactory.getProperties();

           for ( Iterator it = properties.entrySet().iterator(); it.hasNext(); )
           {
               Map.Entry entry = (Map.Entry) it.next();

               System.setProperty( (String) entry.getKey(), (String) entry.getValue() );
           }

           //  /home/drolsham/dev/continuum/ed/continuum-model/target/classes/org/apache/maven/continuum/model
           //URL[] classFiles = new URL[]{getClass().getResource( "/org/apache/maven/continuum/model/project" )};

           URL[] jdoFiles = new URL[]{getClass().getResource( "/META-INF/package.jdo" )};
           URL[] classFiles = new URL[] {};
           SchemaTool.createSchemaTables(jdoFiles, classFiles, null, false, null );
           //SchemaTool.createSchemaTables( new URL[]{getClass().getResource( "/META-INF/package.jdo" )}, new URL[] {}, null, false, null );

           PersistenceManagerFactory pmf = jdoFactory.getPersistenceManagerFactory();

           assertNotNull( pmf );

           PersistenceManager pm = pmf.getPersistenceManager();

           pm.close();

           return (ContinuumStore) lookup( ContinuumStore.ROLE );
       }
   }
   */

}
